-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : datatb.vhd
-- Revision : V1.0
-- Author :   Josef Nouzk
-- Created :  22. 04. 2007
--
-- Modifications
-- 1.0	22. 04. 2007 by Josef Nouzk
-----------------------------------------------------------------
-- testbench pro datovou st
-----------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;

ENTITY datatb_vhd IS
END datatb_vhd;

ARCHITECTURE behavior OF datatb_vhd IS 

	-- Component Declaration for the Unit Under Test (UUT)
	COMPONENT data
	PORT(
		sw : IN std_logic_vector(7 downto 0);
		setA : IN std_logic;
		setB : IN std_logic;
		shrA : IN std_logic;
		shlB : IN std_logic;
		resV : IN std_logic;
		load : IN std_logic;
		k1 : IN std_logic;
		k2 : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		numA : OUT std_logic_vector(7 downto 0);
		numB : OUT std_logic_vector(7 downto 0);
		vys : OUT std_logic_vector(15 downto 0);
		zeroA : OUT std_logic;
		zero0 : OUT std_logic
		);
	END COMPONENT;

	--Inputs
	SIGNAL setA :  std_logic := '0';
	SIGNAL setB :  std_logic := '0';
	SIGNAL shrA :  std_logic := '0';
	SIGNAL shlB :  std_logic := '0';
	SIGNAL resV :  std_logic := '0';
	SIGNAL load :  std_logic := '0';
	SIGNAL k1 :  std_logic := '0';
	SIGNAL k2 :  std_logic := '0';
	SIGNAL clk :  std_logic := '0';
	SIGNAL rst :  std_logic := '0';
	SIGNAL sw :  std_logic_vector(7 downto 0) := (others=>'0');

	--Outputs
	SIGNAL numA :  std_logic_vector(7 downto 0);
	SIGNAL numB :  std_logic_vector(7 downto 0);
	SIGNAL vys :  std_logic_vector(15 downto 0);
	SIGNAL zeroA :  std_logic;
	SIGNAL zero0 :  std_logic;

BEGIN

	-- Instantiate the Unit Under Test (UUT)
	uut: data PORT MAP(
		sw => sw,
		setA => setA,
		setB => setB,
		shrA => shrA,
		shlB => shlB,
		resV => resV,
		load => load,
		k1 => k1,
		k2 => k2,
		numA => numA,
		numB => numB,
		vys => vys,
		zeroA => zeroA,
		zero0 => zero0,
		clk => clk,
		rst => rst
	);

	process
		begin
			clk<='0';
			wait for 10ns;
			clk<='1';
			wait for 10ns;
	end process;


	tb : PROCESS
	
	variable err_cnt: integer :=0;
	
	BEGIN

		-- Wait 100 ns for global reset to finish
		wait for 100 ns;
		rst <= '1';
		wait for 40ns;
		rst <= '0';
		wait for 40ns;
		-- Place stimulus here
		
		sw <= "01010101";
		wait for 80ns;
		setA <= '1';
		wait for 80ns;
		setA <='0';
		wait for 80ns;
		setB <='1';
		wait for 80ns;
		setB <='0';
		wait for 80ns;
		assert (numA="01010101" and numB="01010101") report "shl Error!" severity error;
		if (numA/="01010101" and numB/="01010101") then
			err_cnt:=err_cnt+1;
		end if;
		
		
		----vysledek testu----------
		if (err_cnt=0) then
	    assert false
	    report "Testbench of Data completed successfully!"
	    severity note;
		else
	    assert true
	    report "Something wrong, try again"
	    severity error;
		end if;

		wait; -- will wait forever
	END PROCESS;

END;
