-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : ctrl.vhd
-- Revision : V1.0
-- Author :   Zdenk Sojka
-- Created :  20. 04. 2007
--
-- Modifications
-- 1.0	22. 04. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- adi, dic st, obsahuje instance soust
--
--	vstupy	: od uivatele
--				  stavov signly od datov sti
-- vstupy	: dic signly pro datovou st
--
-- 1.0	prava itelnosti kdu
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ctrl is
	port (
		zeroA : in std_logic;							-- priznak - registr A nulovy
		zero0 : in std_logic;							-- priznak - posledni bit registru A nulovy

		btnA : in std_logic;								-- prikaz - nastaveni 'numA', 'regA' podle 'sw'
		btnB : in std_logic;								-- prikaz - nastaveni 'numB', 'regB' podle 'sw'
		
		setA : out std_logic;							-- prikaz - nastaveni 'numA', 'regA' podle 'sw'
		setB : out std_logic;							-- prikaz - nastaveni 'numB', 'regB' podle 'sw'
		shrA : out std_logic;							-- prikaz - regA >> 1
		shlB : out std_logic;							-- prikaz - regB << 1
		resV : out std_logic;								-- prikaz - nulovani registru s vysledkem
		load : out std_logic;								-- prikaz - nacteni pracovnich registru z ulozenych hodnot
		
		k1 : out std_logic;								-- scitani, prvni krok
		k2 : out std_logic;								-- druhy krok scitani - signal pro scitacku pro zachovani priznaku

		clk : in std_logic;
		rst : in std_logic
	);

end ctrl;

architecture rtl of ctrl is

signal i_op : std_logic_vector(11 downto 0);
signal i_naddr : std_logic_vector(2 downto 0);
signal i_addr : std_logic_vector(2 downto 0);
signal i_vp : std_logic_vector(1 downto 0);

signal i_k2 : std_logic;
signal i_k1 : std_logic;
signal i_shlB : std_logic;
signal i_shrA : std_logic;
signal i_setB : std_logic;
signal i_setA : std_logic;

component mem_ctrl		-- registrovane vstupy !
	port (
		addr : in std_logic_vector(2 downto 0);
		data : out std_logic_vector(11 downto 0)
		);
end component;

component opdekod
	port (
		op : in std_logic_vector(11 downto 0);

		naddr : out std_logic_vector(2 downto 0);
		vp : out std_logic_vector(1 downto 0);

		k2 : out std_logic;
		k1 : out std_logic;
		resV : out std_logic;
		load : out std_logic;
		shlB : out std_logic;
		shrA : out std_logic;
		setB : out std_logic;
		setA : out std_logic);
end component;

component naddr
	port (
		naddr : in std_logic_vector(2 downto 0);
		vp : in std_logic_vector(1 downto 0);
		btnA : in std_logic;
		btnB : in std_logic;
		zero0 : in std_logic;
		zeroA : in std_logic;
		addr : out std_logic_vector(2 downto 0);
		rst : in std_logic;
		clk : in std_logic);
end component;

begin

	mem : mem_ctrl port map (
		addr => i_addr,
		data => i_op
	);
	
	opdek : opdekod port map (
		op => i_op,
		naddr => i_naddr, vp => i_vp,
		k2 => k2, k1 => k1, shlB => shlB, shrA => shrA, setB => setB, setA => setA, resV => resV, load => load
	);

	n_addr : naddr port map (
		naddr => i_naddr, vp => i_vp, btnA => btnA, btnB => btnB, zero0 => zero0, zeroA => zeroA,
		rst => rst, clk => clk,
		addr => i_addr
	);

end rtl;

