-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : top.vhd
-- Revision : V1.0
-- Author :   Zdenk Sojka
-- Created :  20. 04. 2007
--
-- Modifications
-- 1.0	22. 04. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- top soubor k celmu projektu
--
-- 1.0	prava itelnosti kdu
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
entity top is
  port(
  -- dio2 external signals
 cs, we, oe : OUT std_logic;                      -- CPLD drive signals
       addr : OUT std_logic_vector(6 DOWNTO 0);   -- CPLD device address
       data : INOUT std_logic_vector(7 DOWNTO 0); -- DIO2 data bus
    dio2clk : OUT std_logic;                      -- DIO2 peripherals clock signal
     lcd_rw : OUT std_logic;                      -- LCD read/write
     lcd_rs : OUT std_logic;                      -- LCD function select
      lcd_e : OUT std_logic;                      -- LCD enable

    -- global D2E signals
    clk : in std_logic
  );
end entity top;

architecture rtl of top is

signal clocks_sig    : std_logic_vector(24 downto 0);
signal res_sig       : std_logic; -- internal reset
signal sseg_sig      : std_logic_vector(15 DOWNTO 0); -- 7 segment data
signal dio_sig       : std_logic_vector(15 DOWNTO 0); -- Diodes data
signal btns_sig      : std_logic_vector(14 DOWNTO 0); -- 15 Buttons
signal swtch_sig     : std_logic_vector(7 DOWNTO 0);  -- 8 Switches
signal lcd_on_sig    : std_logic;
signal lcd_sig       : std_logic_vector(63 downto 0);    -- 16 x 4bits to display on LCD

component dio2_interface
  PORT( clk : IN std_logic;                       -- clock signal
       reset: IN std_logic;                       -- global async. reset 
       -- dio2 external signals
 cs, we, oe : OUT std_logic;                      -- CPLD drive signals
       addr : OUT std_logic_vector(6 DOWNTO 0);   -- CPLD device address
       data : INOUT std_logic_vector(7 DOWNTO 0); -- DIO2 data bus
    dio2clk : OUT std_logic;                      -- DIO2 peripherals clock signal
     lcd_rw : OUT std_logic;                      -- LCD read/write
     lcd_rs : OUT std_logic;                      -- LCD function select
      lcd_e : OUT std_logic;                      -- LCD enable
      -- internal peripheral interface
  clocks    : OUT std_logic_vector(24 downto 0);
  res       : OUT std_logic; -- internal reset
  sseg      : IN  std_logic_vector(15 DOWNTO 0); -- 7 segment data
  dio       : IN  std_logic_vector(15 DOWNTO 0); -- Diodes data
  btns      : OUT std_logic_vector(14 DOWNTO 0); -- 15 Buttons
  swtch     : OUT std_logic_vector(7 DOWNTO 0);  -- 8 Switches
  lcd_on    : IN  std_logic;
  lcd       : IN  std_logic_vector(63 downto 0)    -- 16 x 4bits to display on LCD
       );
END component dio2_interface;

component proc is
	port (
		btns : in std_logic_vector(1 downto 0);	-- tlatka z DIO
		swtch : in std_logic_vector(7 downto 0);	-- switche z DIO

		numA : out std_logic_vector(7 downto 0);	-- prvn initel
		numB : out std_logic_vector(7 downto 0);	-- druh initel
		vys : out std_logic_vector(15 downto 0);	-- souin

		clk : in std_logic;
		rst : in std_logic);
end component proc;

begin

i_dio2_interface : dio2_interface
  port map ( 
  
    clk => clk,
    reset => '0',
  
    cs => cs,
    we => we,
    oe => oe,
    addr => addr,
    data => data,
    dio2clk => dio2clk,
    lcd_rw => lcd_rw,
    lcd_rs => lcd_rs,
    lcd_e => lcd_e,

    clocks => clocks_sig,
    res => res_sig,
    sseg => sseg_sig,
    dio => dio_sig,
    btns => btns_sig,
    swtch => swtch_sig,
    lcd_on => lcd_on_sig,
    lcd => lcd_sig
  );

lcd_on_sig <= '1';

seme1 : proc
	port map (
		btns => btns_sig(1 downto 0),
		swtch => swtch_sig,
		numA => sseg_sig(7 downto 0),
		numB => sseg_sig(15 downto 8),
		vys => lcd_sig(15 downto 0),

		-- global D2E signals
		rst => res_sig,
		clk => clk
  );

end rtl;

