-----------------------------------------------------------------
-- (c) 2007 VUT-FEL, K336
-- http://cs.felk.cvut.cz
-----------------------------------------------------------------
-- Filename : data.vhd
-- Revision : V1.0
-- Author :   Zdenk Sojka
-- Created :  18. 04. 2007
--
-- Modifications
-- 1.0	22. 04. 2007 by Zdenk Sojka
-----------------------------------------------------------------
-- datov st
-- obsahuje registry a poetn logiku
--
-- vstupy	: pkazy z dic sti
--				  nastaven switch jako vstup do registr
-- vstupy	: stavov signly
--				  data pro zobrazen na LSD a sseg
--
-- 1.0	prava itelnosti kdu
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity data is
	Port (
		sw : in std_logic_vector(7 downto 0);		-- 8 bit hodnota pri zapisu do registru (nastaveni switchu)

		setA : in std_logic;								-- prikaz - nastaveni 'numA', 'regA' podle 'sw'
		setB : in std_logic;								-- prikaz - nastaveni 'numB', 'regB' podle 'sw'
		shrA : in std_logic;								-- prikaz - regA >> 1
		shlB : in std_logic;								-- prikaz - regB << 1
		resV : in std_logic;								-- prikaz - nulovani registru s vysledkem
		load : in std_logic;								-- prikaz - nacteni pracovnich registru z ulozenych hodnot
		
		k1 : in std_logic;								-- prikaz - scitani, prvni krok
		k2 : in std_logic;								-- prikaz - druhy krok scitani - signal pro scitacku pro zachovani priznaku

      numA : out std_logic_vector(7 downto 0);	-- prvni cislo pro zobrazeni na 8seg
      numB : out std_logic_vector(7 downto 0);	-- druhe cislo -//-
      vys : out std_logic_vector(15 downto 0);	-- soucin pro zobrazeni na LCD

		zeroA : out std_logic;							-- priznak - registr A nulovy
		zero0 : out std_logic;							-- priznak - posledni bit registru A nulovy

		clk : in std_logic;
		rst : in std_logic
	);
end data;

architecture rtl of data is

--signal cnt : std_logic_vector(2 downto 0);		-- citaci registr - neni potreba
--signal regA : std_logic_vector(7 downto 0);		-- prvni cinitel
signal valB_r : std_logic_vector(15 downto 0);		-- druhy cinitel
signal vys_i : std_logic_vector(15 downto 0);	-- soucin

signal in1 : std_logic_vector(7 downto 0);		-- prvni scitanec do scitacky
signal in2 : std_logic_vector(7 downto 0);		-- druhy
signal sum : std_logic_vector(7 downto 0);		-- soucet

signal vys_nxt : std_logic_vector(15 downto 0);

signal numA_i : std_logic_vector(7 downto 0);
signal numB_i : std_logic_vector(7 downto 0);

signal valA_i : std_logic_vector(7 downto 0);
signal valB_i : std_logic_vector(15 downto 0);

signal setAf : std_logic;
signal setBf : std_logic;

component add8cz
	Port ( in1, in2 : in std_logic_vector ( 7 downto 0 );
			k2, clk, rst : in std_logic;
			sum : out std_logic_vector ( 7 downto 0 )
			);
end component;

component registrA
   port (
		sw		: in	std_logic_vector (7 downto 0);
		set	: in	std_logic;
		shr	: in	std_logic;
		zero	: out std_logic;
		zero0	: out std_logic;

		clk	: in	std_logic;
		rst	: in	std_logic );
end component;

component registrB
   port (
		sw		: in	std_logic_vector (15 downto 0);
		set	: in	std_logic;
		shl	: in	std_logic;

		val	: out	std_logic_vector (15 downto 0);

		clk	: in	std_logic;
		rst	: in	std_logic );

end component;

begin

	setAf <= setA or load;							-- nastaven pi setA od uivatele, pi load z numA_i

	valA_i <= (others=>'0') when rst='1' else
		numA_i when load='1' else
		sw;

	regA : registrA port map (
		sw => valA_i, set => setAf, shr => shrA,
		clk => clk, rst => rst,
		zero => zeroA, zero0 => zero0
	);

	setBf <= setB or load;							-- -//-

	valB_i <= (others=>'0') when rst='1' else
		"00000000" & numB_i when load='1' else
		"00000000" & sw;

	regB : registrB port map (
		sw => valB_i, set => setBf, shl => shlB,
		clk => clk, rst => rst,
		val => valB_r
	);

	in1 <= valB_r( 15 downto 8 ) when k2 = '1' else	-- prvn stanec (registr B)
			valB_r( 7 downto 0 );								-- pokud nen druh krok, st se doln polovina

	in2 <= vys_i( 15 downto 8 ) when k2 = '1' else	-- druh stanec
			vys_i( 7 downto 0 );

	scitacka : add8cz port map (
		in1 => in1, in2 => in2, k2 => k2,
		clk => clk, rst => rst,
		sum => sum
	);
	
	vys_nxt <= (others=>'0') when (rst='1' or resV='1') else	-- reset nebo pkaz resetu vsledku
			sum & vys_i( 7 downto 0 ) when k2 = '1' else		-- 2. krok - nastaven horn sti registru
			vys_i( 15 downto 8 ) & sum when k1 = '1' else	-- 1. krok - nastaven doln sti registru
			vys_i;														-- jinak zachovn star hodnoty

	zpracovani : process ( clk, rst ) is

	begin
		if ( rst = '1' ) then
			numA_i <= (others=>'0');		-- nastaveni hezkych hodnot pri restartu pripravku
			numB_i <= (others=>'0');
			vys_i <= (others=>'0');
		elsif ( clk = '1' ) and clk'event then
			if setA = '1' then
				numA_i <= sw;
			end if;
			if setB = '1' then
				numB_i <= sw;
			end if;
			vys_i <= vys_nxt;
		end if;
	end process;
	
	vys <= vys_i;
	numA <= numA_i;
	numB <= numB_i;

end rtl;

